"use strict";

// Import dependencies
import("/js/core.js").then(() =>
	import("/js/damper.js").then(() => {
		const dependencies = setInterval(() => {
			if (
				typeof core === "function" &&
				typeof core.damper === "function"
			) {
				clearInterval(dependencies);
				clearTimeout(timeout);
				initialization();
			}
		}, 10);
		const timeout = setTimeout(() => {
			clearInterval(dependencies);
			initialization();
		}, 5000);

		function initialization() {
			if (typeof core.cart === "undefined") {
				// Not initialized

				/**
				 * @name Cart
				 *
				 * @description
				 * Implements actions with cart
				 *
				 * @memberof core
				 *
				 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
				 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
				 */
				core.cart = class cart {
					/**
					 * Toggle
					 *
					 * Toggle the product in the cart (interface)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static toggle(button, force = false) {
						// Blocking the button
						button.setAttribute("disabled", "true");

						// Execute under damper
						this.toggle_damper(button, force);

						// Exit (success)
						return false;
					}

					/**
					 * Toggle
					 *
					 * Toggle the product in the cart (damper)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {void}
					 */
					static toggle_damper = core.damper(
						(...variables) => this.toggle_system(...variables),
						300,
						2,
					);

					/**
					 * Toggle
					 *
					 * Toggle the product in the cart (system)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 *
					 * @return {Promise} Request to the server
					 *
					 * @todo add unblocking button by timer + everywhere
					 */
					static async toggle_system(button) {
						if (button instanceof HTMLElement) {
							// Validated

							// Initializing of the wrap of buttons
							const wrap = button.parentElement;

							// Initializing of the product
							const product = wrap.parentElement;

							// Initializing of identifier of the product
							const identifier = product.getAttribute(
								"data-product-identifier",
							);

							if (typeof identifier === "string" && identifier.length > 0) {
								// Validated identifier

								return await core.request(
									"/cart/product",
									`identifier=${identifier}&type=toggle`,
								)
									.then((json) => {
										if (
											json.errors !== null &&
											typeof json.errors === "object" &&
											json.errors.length > 0
										) {
											// Fail (received errors)
										} else {
											// Success (not received errors)

											// Unblocking the button
											button.removeAttribute("disabled");

											// Writing offset of hue-rotate to indicate that the product is in the cart
											wrap.style.setProperty(
												"--hue-rotate-offset",
												json.amount + '0deg',
											);

											// Writing attribute with amount of the product in the cart
											product.setAttribute(
												"data-product-amount",
												json.amount,
											);

											// Initializing the amount <span> element
											const amount = wrap.querySelector(
												'span[data-product-button-text="amount"]',
											);

											if (amount instanceof HTMLElement) {
												// Initialized the amount <span> element

												// Writing amount of the product in the cart
												amount.innerText = json.amount;
											}
										}
									});
							}
						}
					}

					/**
					 * Write
					 *
					 * Write the product in the cart (interface)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of writings
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static write(button, amount = 1, force = false) {
						// Blocking the button
						button.setAttribute("disabled", "true");

						// Execute under damper
						this.write_damper(button, amount, force);

						// Exit (success)
						return false;
					}

					/**
					 * Write
					 *
					 * Write the product in the cart (damper)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of writings
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {void}
					 */
					static write_damper = core.damper(
						(...variables) => this.write_system(...variables),
						300,
						3,
					);

					/**
					 * Write
					 *
					 * Write the product in the cart (system)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of writings
					 *
					 * @return {Promise} Request to the server
					 *
					 * @todo add unblocking button by timer + everywhere
					 */
					static async write_system(button, amount = 1) {
						if (
							button instanceof HTMLElement &&
							typeof amount === "number" &&
							amount > -1 &&
							amount < 100
						) {
							// Validated

							// Initializing of the wrap of buttons
							const wrap = button.parentElement;

							// Initializing of the product
							const product = wrap.parentElement;

							// Initializing of identifier of the product
							const identifier = product.getAttribute(
								"data-product-identifier",
							);

							if (typeof identifier === "string" && identifier.length > 0) {
								// Validated identifier

								return await core.request(
									"/cart/product",
									`identifier=${identifier}&type=write&amount=${amount}`,
								)
									.then((json) => {
										if (
											json.errors !== null &&
											typeof json.errors === "object" &&
											json.errors.length > 0
										) {
											// Fail (received errors)
										} else {
											// Success (not received errors)

											// Unblocking the button
											button.removeAttribute("disabled");

											// Writing offset of hue-rotate to indicate that the product is in the cart
											wrap.style.setProperty(
												"--hue-rotate-offset",
												json.amount + '0deg',
											);

											// Writing attribute with amount of the product in the cart
											product.setAttribute(
												"data-product-amount",
												json.amount,
											);

											// Initializing the amount <span> element
											const amount = wrap.querySelector(
												'span[data-product-button-text="amount"]',
											);

											if (amount instanceof HTMLElement) {
												// Initialized the amount <span> element

												// Writing amount of the product in the cart
												amount.innerText = json.amount;
											}
										}
									});
							}
						}
					}

					/**
					 * Delete
					 *
					 * Delete the product from the cart (interface)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of deletings
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static delete(button, amount = 1, force = false) {
						// Blocking the button
						button.setAttribute("disabled", "true");

						// Execute under damper
						this.delete_damper(button, amount, force);

						// Exit (success)
						return false;
					}

					/**
					 * Delete
					 *
					 * Delete the product from the cart (damper)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of deletings
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {void}
					 */
					static delete_damper = core.damper(
						(...variables) => this.delete_system(...variables),
						300,
						3,
					);

					/**
					 * Delete
					 *
					 * Delete the product from the cart (system)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of deletings
					 *
					 * @return {Promise} Request to the server
					 *
					 * @todo add unblocking button by timer + everywhere
					 */
					static async delete_system(button, amount = 1) {
						if (
							button instanceof HTMLElement &&
							typeof amount === "number" &&
							amount > 0 &&
							amount < 101
						) {
							// Validated

							// Initializing of the wrap of buttons
							const wrap = button.parentElement;

							// Initializing of the product
							const product = wrap.parentElement;

							// Initializing of identifier of the product
							const identifier = product.getAttribute(
								"data-product-identifier",
							);

							if (typeof identifier === "string" && identifier.length > 0) {
								// Validated identifier

								return await core.request(
									"/cart/product",
									`identifier=${identifier}&type=delete&amount=${amount}`,
								)
									.then((json) => {
										if (
											json.errors !== null &&
											typeof json.errors === "object" &&
											json.errors.length > 0
										) {
											// Fail (received errors)
										} else {
											// Success (not received errors)

											// Unblocking the button
											button.removeAttribute("disabled");

											// Writing offset of hue-rotate to indicate that the product is in the cart
											wrap.style.setProperty(
												"--hue-rotate-offset",
												json.amount + '0deg',
											);

											// Writing attribute with amount of the product in the cart
											product.setAttribute(
												"data-product-amount",
												json.amount,
											);

											// Initializing the amount <span> element
											const amount = wrap.querySelector(
												'span[data-product-button-text="amount"]',
											);

											if (amount instanceof HTMLElement) {
												// Initialized the amount <span> element

												// Writing amount of the product in the cart
												amount.innerText = json.amount;
											}
										}
									});
							}
						}
					}

					/**
					 * Set
					 *
					 * Set amount of the product in the cart (interface)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of the product in the cart to be setted
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {bool} True if an error occurs to continue the event execution
					 */
					static set(button, amount = 1, force = false) {
						// Blocking the button
						button.setAttribute("disabled", "true");

						// Execute under damper
						this.set_damper(button, amount, force);

						// Exit (success)
						return false;
					}

					/**
					 * Set
					 *
					 * Set the product in the cart (damper)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of the product in the cart to be setted
					 * @param {bool} force Ignore the damper? (false)
					 *
					 * @return {void}
					 */
					static set_damper = core.damper(
						(...variables) => this.set_system(...variables),
						300,
						3,
					);

					/**
					 * Set
					 *
					 * Set the product in the cart (system)
					 *
					 * @param {HTMLElement} button Button of the product <a>
					 * @param {number} amount Amount of the product in the cart to be setted
					 *
					 * @return {Promise} Request to the server
					 *
					 * @todo add unblocking button by timer + everywhere
					 */
					static async set_system(button, amount = 1) {
						if (
							button instanceof HTMLElement &&
							typeof amount === "number" &&
							amount > -1 &&
							amount < 101
						) {
							// Validated

							// Initializing of the wrap of buttons
							const wrap = button.parentElement;

							// Initializing of the product
							const product = wrap.parentElement;

							// Initializing of identifier of the product
							const identifier = product.getAttribute(
								"data-product-identifier",
							);

							if (typeof identifier === "string" && identifier.length > 0) {
								// Validated identifier

								return await core.request(
									"/cart/product",
									`identifier=${identifier}&type=set&amount=${amount}`,
								)
									.then((json) => {
										if (
											json.errors !== null &&
											typeof json.errors === "object" &&
											json.errors.length > 0
										) {
											// Fail (received errors)
										} else {
											// Success (not received errors)

											// Unblocking the button
											button.removeAttribute("disabled");

											// Writing offset of hue-rotate to indicate that the product is in the cart
											wrap.style.setProperty(
												"--hue-rotate-offset",
												json.amount + '0deg',
											);

											// Writing attribute with amount of the product in the cart
											product.setAttribute(
												"data-product-amount",
												json.amount,
											);

											// Initializing the amount <span> element
											const amount = wrap.querySelector(
												'span[data-product-button-text="amount"]',
											);

											if (amount instanceof HTMLElement) {
												// Initialized the amount <span> element

												// Writing amount of the product in the cart
												amount.innerText = json.amount;
											}
										}
									});
							}
						}
					}
				};
			}
		}
	})
);
